/*****************************************************************

	sailor.c

	(c) 1995 Tucker Balch

	A program to test and evaluate navigational strategies.
	See docs/sailor.pdf for a more complete description.

//
// Updated 1/23/2002 by Andrew Stein to allow sailor to see the goal 
//  when adjacent to it
//

*****************************************************************/

#include <curses.h>
#include <math.h>
#include "sailor.h"

WINDOW 	*game_win;

int	goal_x, goal_y;
int	sailor_x, sailor_y;
int	obstacles[MAX_X][MAX_Y];


/******************************************************************

	init() 

******************************************************************/
int init()
{
char	c;
int	i, j, val, r;
	
wclear(game_win);

/*
 * Clear out the obstacles map.
 */
for(i=0; i<MAX_X; i++)
	{
	for(j=0; j<MAX_Y; j++)
		{
		obstacles[i][j] = EMPTY;
		}
	}

/*
 * Read the descripton of the world
 * from a file.
 */
c = ' ';
i = 1;
j = 1;
r = scanf("%c", &c);
while (r != EOF)
	{
	while ((c != '\n') && (r != EOF))
		{

		/*--- X marks the spot (home) ---*/
		if(c == 'X')
			{
			goal_x = i;
			goal_y = j;
			obstacles[i][j] = GOAL;  // added 1/23/2002 - ANS
			}

		/*--- Location of sailor ---*/
		if(c == '*')
			{
			sailor_x = i;
			sailor_y = j;
			}

		/*--- An obstacle ---*/
		if(c == 'O')
			obstacles[i][j] = OCCUPIED;

		mvaddch(j, i, c); /* display it */
		i++;
		if(i==(MAX_X)) i--;
		r = scanf("%c", &c);
		}
	j++;
	i = 0;
	if(j==(MAX_Y)) j--;
	c = 'O';
	}

/*
 * Put obstacles around the boundary.  
 * That will keep the sailor in bounds all the time.
 */
for(i=0; i<MAX_X; i++)
	{
	for(j=0; j<MAX_Y; j++)
		{
		if ((j==0)||(j==(MAX_Y-1))||(i==0)||(i==(MAX_X -1)))
			{
			obstacles[i][j] = OCCUPIED;
			mvaddch(j, i, 'O');
			}
		}
	}
wrefresh(game_win);
}
		
    

/*****************************************************************

	main() 

	Sets up the environment and calls navigate.

*****************************************************************/
int main()
{
char 	ch;
int	i,j,k;
int	done = 0;
int	count = 0;
int	local_obstacles[9];
int	ship_direction = N;
int	new_heading = N;
double	temp_angle = 0;
double	path = 0;

initscr();
game_win = subwin(stdscr, LINES, COLS, 0, 0);

wmove(game_win,0,0);
wrefresh(game_win);
init();

sleep(3);
while (done != 1)
	{
	count++;
	
	/*
	 * Note the local obstacles
	 */
	k = 0;
	for (j = 0; j < 3; j++)
		for (i = 0; i < 3; i++)
			{
			local_obstacles[k++] = 
				obstacles[sailor_x+i-1][sailor_y+j-1];
			}
	
	/*
	 * Figure out heading to ship
	 */
	temp_angle = atan2((goal_x - sailor_x), (goal_y - sailor_y));
	temp_angle += PI;
	temp_angle = 360*temp_angle/(2*PI);
	ship_direction = N;
	if (temp_angle > 22.5+0*45)
		ship_direction = NW;
	if (temp_angle > 22.5+1*45)
		ship_direction = W;
	if (temp_angle > 22.5+2*45)
		ship_direction = SW;
	if (temp_angle > 22.5+3*45)
		ship_direction = S;
	if (temp_angle > 22.5+4*45)
		ship_direction = SE;
	if (temp_angle > 22.5+5*45)
		ship_direction = E;
	if (temp_angle > 22.5+6*45)
		ship_direction = NE;
	if (temp_angle > 22.5+7*45)
		ship_direction = N;

	/*
	 * Find out which way to go.
	 */
	new_heading = navigate(local_obstacles, ship_direction);
	if ((new_heading < 0) || (new_heading > 8))
		{
		fprintf(stderr, "illegal heading: %d\n", new_heading);
		new_heading = N;
		}

	/*
	 * Move the sailor in the world
	 */
	mvaddch(sailor_y, sailor_x, '.'); /* leave a . trail */
	if (local_obstacles[new_heading] != OCCUPIED) // changed from "== EMPTY" on 1/23/2002 - ANS
		{
		switch(new_heading)
			{
			case N: sailor_y--; path++; break;
			case S: sailor_y++; path++; break;
			case E: sailor_x++; path++; break;
			case W: sailor_x--; path++; break;
			case NE: sailor_y--; sailor_x++; path+=SQR2; break;
			case NW: sailor_y--; sailor_x--; path+=SQR2; break;
			case SE: sailor_y++; sailor_x++; path+=SQR2; break;
			case SW: sailor_y++; sailor_x--; path+=SQR2; break;
			}
		}
	mvaddch(sailor_y, sailor_x, '*'); /* new position */
	wrefresh(game_win);
	if ((sailor_x == goal_x) && (sailor_y == goal_y))
		done = 1;
	if (count > TIME_LIMIT)
		done = 1;
	}

/*
 * Report the results and exit.
 */
for(i=0; i<MAX_Y; i++) /* go to the bottom of the page */
	printf("\n");
if (count > TIME_LIMIT)
	{
	printf("timeout\n");
	}
else
	{
	printf("path length	: %f\f\r", path);
	printf("time steps 	: %d\f\r", count);
	printf("\n\f\r");
	}

reset_shell_mode();
return 0;
}

